@@ -1,13 +1,351 @@
 import numpy as np
-try:
-    from unitSystem import _unitConversion, knownUnits, knownCharacters, knownUnitsDict, knownPrefixes, baseUnit
-except ImportError:
-    from pyees.unitSystem import _unitConversion, knownUnits, knownCharacters, knownUnitsDict, knownPrefixes, baseUnit
 
+
+class _unitConversion():
+
+    def __init__(self, scale, offset=0) -> None:
+        self.scale = scale
+        self.offset = offset
+
+    def __mul__(self, other):
+        if isinstance(other, _unitConversion):
+            scale = self.scale * other.scale
+            offset = self.scale * other.offset + self.offset
+        else:
+            scale = self.scale * other
+            offset = self.offset
+        return _unitConversion(scale, offset)
+
+    def __pow__(self, other):
+        scale = 1
+        offset = 0
+        
+        for _ in range(other):
+            scale *= self.scale
+            offset*= self.scale
+            offset += self.offset
+
+        return _unitConversion(scale, offset)
+
+    def __truediv__(self, other):
+        if isinstance(other, _unitConversion):
+            return _unitConversion(1 / other.scale, - other.offset / other.scale) * self
+        return self * _unitConversion(1 / other)
+
+
+    def convert(self, value, useOffset=True):
+        if useOffset:
+            return self.scale * value + self.offset
+        else:
+            return self.scale * value
+
+class neperConversion():
+    def __init__(self):
+        self.scale = 1
+        self.offset = 0
+    @staticmethod
+    def convertToSignal(var):
+        var._uncert = 2*np.exp(2*var.value) * var.uncert
+        var._value = np.exp(2*var.value)
+    @staticmethod
+    def convertFromSignal(var):
+        var._uncert = 1 / (2*var.value) * var.uncert
+        var._value = 1/2 * np.log(var.value)
+    def __mul__(self, other):
+        return 1 * other
+    def __rmul__(self, other):
+        return self * other
+    def __truediv__(self, other):
+        return 1 / other
+    def __rtruediv__(self, other):
+        return other / 1
+    def __pow__ (self, other):
+        return 1** other
+    def __rpow__(self, other):
+        return other ** 1
+
+class bellConversion():
+    def __init__(self):
+        self.scale = 1
+        self.offset = 0
+    @staticmethod
+    def convertToSignal(var):
+        var._uncert = 10**var.value * np.log(10) * var.uncert
+        var._value = 10**var.value
+    @staticmethod
+    def convertFromSignal(var):
+        var._uncert = 1 / (var.value * np.log(10)) * var.uncert
+        var._value = np.log10(var.value)
+    def __mul__(self, other):
+        return 1 * other
+    def __rmul__(self, other):
+        return self * other
+    def __truediv__(self, other):
+        return 1 / other
+    def __rtruediv__(self, other):
+        return other / 1
+    def __pow__ (self, other):
+        return 1** other
+    def __rpow__(self, other):
+        return other ** 1
+
+class octaveConversion():
+    def __init__(self):
+        self.scale = 1
+        self.offset = 0
+    @staticmethod
+    def convertToSignal(var):
+        var._uncert = 2**var.value * np.log(2) * var.uncert
+        var._value = 2**var.value
+    @staticmethod
+    def convertFromSignal(var):
+        var._uncert = 1 / (var.value * np.log(2)) * var.uncert
+        var._value = np.log2(var.value)
+    def __mul__(self, other):
+        return 1 * other
+    def __rmul__(self, other):
+        return self * other
+    def __truediv__(self, other):
+        return 1 / other
+    def __rtruediv__(self, other):
+        return other / 1
+    def __pow__ (self, other):
+        return 1** other
+    def __rpow__(self, other):
+        return other ** 1
+
+    
+baseUnit = {
+    '1': _unitConversion(1),
+    '': _unitConversion(1),
+    '%': _unitConversion(1e-2)
+}
+
+force = {
+    'N': _unitConversion(1)
+}
+
+mass = {
+    'g': _unitConversion(1 / 1000)
+}
+
+energy = {
+    'J': _unitConversion(1),
+}
+
+power = {
+    'W': _unitConversion(1)
+}
+
+pressure = {
+    'Pa': _unitConversion(1),
+    'bar': _unitConversion(1e5)
+}
+
+temperature = {
+    'K': _unitConversion(1),
+    'C': _unitConversion(1, 273.15),
+    'F': _unitConversion(5 / 9, 273.15 - 32 * 5 / 9)
+}
+
+temperatureDifference = {
+    'DELTAK': _unitConversion(1),
+    'DELTAC': _unitConversion(1),
+    'DELTAF': _unitConversion(5 / 9)
+}
+
+time = {
+    's': _unitConversion(1),
+    'min': _unitConversion(60),
+    'h': _unitConversion(60 * 60),
+    'yr': _unitConversion(60 * 60 * 24 * 365)
+}
+
+volume = {
+    'm3': _unitConversion(1),
+    'L': _unitConversion(1 / 1000)
+}
+
+length = {
+    'm': _unitConversion(1),
+    'Å': _unitConversion(1e-10),
+    'ly': _unitConversion(9460730472580800)
+}
+
+angle = {
+    'rad': _unitConversion(1),
+    'deg': _unitConversion(np.pi / 180)
+}
+
+current = {
+    'A': _unitConversion(1)
+}
+
+voltage = {
+    'V': _unitConversion(1)
+}
+
+frequency = {
+    'Hz': _unitConversion(1)
+}
+
+resistance = {
+    'ohm': _unitConversion(1)
+}
+
+kinematicViscosity = {
+    'St': _unitConversion(1e-4)
+}
+
+logrithmicUnits = {
+    'Np' : neperConversion(),
+    'B': bellConversion(),
+    'oct': octaveConversion(),
+    'dec': bellConversion()
+}
+
+knownUnitsDict = {
+    'kg-m/s2': force,
+    'kg/m-s2': pressure,
+    's': time,
+    'K': temperature,
+    'DELTAK': temperatureDifference,
+    'm3': volume,
+    'm': length,
+    'kg-m2/s2': energy,
+    'kg-m2/s3': power,
+    'kg': mass,
+    'A': current,
+    'kg-m2/s3-A': voltage,
+    '1': baseUnit,
+    '1/s': frequency,
+    'rad': angle,
+    'kg-m2/s3-A2' : resistance,
+    'm2/s' : kinematicViscosity,
+    'Np': logrithmicUnits
+}
+
+knownPrefixes = {
+    'T': 1e12,
+    'G': 1e9,
+    'M': 1e6,
+    'k': 1e3,
+    'h': 1e2,
+    'da': 1e1,
+    'd': 1e-1,
+    'c': 1e-2,
+    'm': 1e-3,
+    'mu': 1e-6,
+    'n': 1e-9,
+    'p': 1e-12
+}
+
+
+knownUnits = {}
+for key, d in knownUnitsDict.items():
+    for item, _ in d.items():
+        if item not in knownUnits:
+            knownUnits[item] = [key, knownUnitsDict[key][item]]
+        else:
+            raise Warning(f'The unit {item} known in more than one unit system')
+
+
+
+# determine the known characters within the unit system
+knownCharacters = list(knownUnits.keys()) + list(knownPrefixes.keys())
+knownCharacters = ''.join(knownCharacters)
+knownCharacters += '-/'
+knownCharacters += '0123456789'
+knownCharacters += '()'
+knownCharacters = set(knownCharacters)
+
+# check if all unit and prefix combinations can be distiguished
+unitPrefixCombinations = []
+for u in knownUnits:
+    unitPrefixCombinations += [u]
+    if u not in baseUnit or u == "%":
+        unitPrefixCombinations += [p + u for p in knownPrefixes]
+
+for elem in unitPrefixCombinations:
+    count = sum([1 if u == elem else 0 for u in unitPrefixCombinations])
+    if count > 1:
+        prefix = elem[0:1]
+        unit = elem[1:]
+
+        unitType1 = ''
+        for key, item in knownUnitsDict.items():
+            if elem in item:
+                unitType1 = [key for key, a in locals().items() if a == item][0]
+        if unitType1 == '':
+            raise ValueError(f'The unit {elem} was not found.')
+
+        unitType2 = ''
+        for key, item in knownUnitsDict.items():
+            if unit in item:
+                unitType2 = [key for key, a in locals().items() if a == item][0]
+        if unitType2 == '':
+            raise ValueError(f'The unit {unit} was not found.')
+
+        raise ValueError(f'The unit {elem} can be interpreted as a {unitType1} or a {unitType2} with the prefix {prefix}. The cannot be distiguished.')
+
+
+
+
+
+def addNewUnit(newUnit: str, scale: float, existingUnit: str, offset : float = 0):
+        
+    if newUnit in unitPrefixCombinations:
+        raise ValueError(f'The unit {newUnit} is already known within the unit system')
+    unitPrefixCombinations.append(newUnit)
+    for p in knownPrefixes:
+        if p+newUnit in unitPrefixCombinations:
+            raise ValueError(f'The unit {p+newUnit} is already known within the unit system')
+        unitPrefixCombinations.append(newUnit)
+
+    ## create the conversion 
+    conversion =  _unitConversion(scale, offset)
+
+
+    upper, upperPrefix,upperExp, lower, lowerPrefix, lowerExp = unit._getLists(existingUnit)
+    nUpper = len(upper)
+    units = upper + lower
+    exponents = upperExp + lowerExp
+    prefixes = upperPrefix + lowerPrefix
+    SIBaseUnitLists = [[],[]]
+    
+    for i, (u, exponent, prefix) in enumerate(zip(units, exponents, prefixes)):
+                
+        _SIBaseUnit, unitConversion = knownUnits[u]
+        unitConversion =  unitConversion ** exponent
+
+        if exponent != 1: _SIBaseUnit += str(exponent)
+        
+        if not prefix is None: unitConversion *= knownPrefixes[prefix]
+
+        if i <= nUpper-1:
+            conversion = unitConversion * conversion
+            SIBaseUnitLists[0].append(_SIBaseUnit)
+        else:
+            conversion = _unitConversion(1) / unitConversion * conversion
+            SIBaseUnitLists[1].append(_SIBaseUnit)
+            
+    SIBaseUnit = '-'.join(SIBaseUnitLists[0])
+    if SIBaseUnitLists[1]: SIBaseUnit += '/' + '-'.join(SIBaseUnitLists[1])
+    
+    knownUnits[newUnit] = [SIBaseUnit, conversion]
+        
+    global knownCharacters
+    for s in newUnit:
+        knownCharacters.add(s)
+
+    
+    
+    
+    
 hyphen = '-'
 slash = '/'
 integers = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
-temperatureUnits = knownUnitsDict['K']
+
 
 class unit():
 
@@ -172,8 +510,8 @@ class unit():
         lower, lowerPrefix, lowerExp = unit._splitUnitExponentAndPrefix(lower)
 
         if lower:
-            upper = ['DELTA' + elem if elem in temperatureUnits else elem for elem in upper]
-            lower = ['DELTA' + elem if elem in temperatureUnits else elem for elem in lower]
+            upper = ['DELTA' + elem if elem in temperature else elem for elem in upper]
+            lower = ['DELTA' + elem if elem in temperature else elem for elem in lower]
 
         return upper, upperPrefix, upperExp, lower, lowerPrefix, lowerExp
 
@@ -552,15 +890,14 @@ class unit():
         return unit._multiply(a, b)
 
     def __pow__(self, power):
-
         if power == 0:
-            return '1'
+            return '1', False
 
         elif power > 1:
 
             if self.unitStr == '1':
                 # self is '1'. Therefore the power does not matter
-                return self.unitStr
+                return self.unitStr, False
 
             else:
                 # self is not '1'. Therefore all exponents are multiplied by the power
@@ -571,7 +908,7 @@ class unit():
                 upperExp = [int(elem * power) for elem in self.upperExp]
                 lowerExp = [int(elem * power) for elem in self.lowerExp]
 
-                return self._combineUpperAndLower(self.upper, self.upperPrefix, upperExp, self.lower, self.lowerPrefix, lowerExp)
+                return self._combineUpperAndLower(self.upper, self.upperPrefix, upperExp, self.lower, self.lowerPrefix, lowerExp) , False
 
         else:
             # the power is smaller than 1.
@@ -579,7 +916,7 @@ class unit():
 
             if self.unitStr == '1':
                 # self is '1'. Therefore the power does not matter
-                return self.unitStr
+                return self.unitStr, False
             else:
                 # self is not '1'.
                 # Therefore it is necessary to determine if all exponents are divisible by the recibricol of the power
@@ -587,16 +924,32 @@ class unit():
                 def isCloseToInteger(a, rel_tol=1e-9, abs_tol=0.0):
                     b = np.around(a)
                     return abs(a - b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)
-
+                
                 # Test if the exponent of all units is divisible by the power
-                for exp in self.upperExp + self.lowerExp:
-                    if not isCloseToInteger(exp * power):
-                        raise ValueError(f'You can not raise a variable with the unit {self.unitStr} to the power of {power}')
+                try:
+                    for exp in self.upperExp + self.lowerExp:
+                        if not isCloseToInteger(exp * power):
+                            raise ValueError(f'You can not raise a variable with the unit {self.unitStr} to the power of {power}')
 
-                upperExp = [int(elem * power) for elem in self.upperExp]
-                lowerExp = [int(elem * power) for elem in self.lowerExp]
+                    upperExp = [int(elem * power) for elem in self.upperExp]
+                    lowerExp = [int(elem * power) for elem in self.lowerExp]
+
+                    return self._combineUpperAndLower(self.upper, self.upperPrefix, upperExp, self.lower, self.lowerPrefix, lowerExp), False
+                
+                except ValueError:                
+                    ## the exponents of the unit was not divisible by the power
+                    ## test if the exponents of the SIBaseunit is divisible by the power 
+                    
+                    siUpper, siUpperPrefix, siUpperExp, siLower, siLowerPrefix, siLowerExp = self._getLists(self._SIBaseUnit)
+                    for exp in siUpperExp + siLowerExp:
+                        if not isCloseToInteger(exp * power):
+                            raise ValueError(f'You can not raise a variable with the unit {self.unitStr} to the power of {power}')
+
+                    siUpperExp = [int(elem * power) for elem in siUpperExp]
+                    siLowerExp = [int(elem * power) for elem in siLowerExp]
+
+                    return self._combineUpperAndLower(siUpper, siUpperPrefix, siUpperExp, siLower, siLowerPrefix, siLowerExp), True
 
-                return self._combineUpperAndLower(self.upper, self.upperPrefix, upperExp, self.lower, self.lowerPrefix, lowerExp)
 
     def getConverter(self, newUnit):
         newUnit = unit._formatUnit(newUnit)
@@ -680,6 +1033,3 @@ class unit():
         self._SIBaseUnit = self._getSIBaseUnit(self.upper, self.upperExp, self.lower, self.lowerExp)
         otherUpper, otherUpperPrefix, otherUpperExp, otherLower, otherLowerPrefix, otherLowerExp = self._getLists(self._SIBaseUnit)
         self._converterToSI = self._getConverter(otherUpper, otherUpperPrefix, otherUpperExp, otherLower, otherLowerPrefix, otherLowerExp)
-
-
-
